@@ -58,35 +58,35 @@ def _createFields( runTime, mesh ):
     laminarTransport = man.singlePhaseTransportModel( U, phi )
     
     turbulence = man.incompressible.turbulenceModel.New( U, phi, laminarTransport )
+    
+    sources = man.IObasicSourceList( mesh )
 
-    return p, U, phi, turbulence, pRefCell, pRefValue, laminarTransport
+    return p, U, phi, turbulence, pRefCell, pRefValue, laminarTransport, sources
 
 
 #--------------------------------------------------------------------------------------
-def Ueqn( mesh, pimple, phi, U, p, turbulence ):
-
-    UEqn = man.fvm.ddt(U) + man.fvm.div(phi, U) + man.fvVectorMatrix( turbulence.divDevReff( U ), man.Deps( turbulence, U ) )
-    
+def Ueqn( mesh, pimple, phi, U, p, turbulence, sources ):
+    # The initial C++ expression does not work properly, because of
+    #  1. turbulence.divDevRhoReff( U ) - changes values for the U boundaries
+    #  2. the order of expression arguments computation differs with C++
+    #UEqn = man.fvm.ddt(U) + man.fvm.div(phi, U) + man.fvVectorMatrix( turbulence.divDevReff( U ), man.Deps( turbulence, U ) )           
+    UEqn = man.fvVectorMatrix( turbulence.divDevReff( U ), man.Deps( turbulence, U ) )  + ( man.fvm.ddt(U) + man.fvm.div(phi, U) )
     UEqn.relax()
+    sources.constrain( UEqn )
     
     rAU = man.volScalarField( 1.0/UEqn.A(), man.Deps( UEqn ) )
 
     if pimple.momentumPredictor():
-       ref.solve( UEqn == -man.fvc.grad( p ) )
-       pass
-    else:
-       U << rAU * ( UEqn.H() - ref.fvc.grad( p ) )
-       U.correctBoundaryConditions()
+       ref.solve( UEqn == -ref.fvc.grad( p ) + sources( U ) )
        pass
     
     return UEqn, rAU
 
 
 #--------------------------------------------------------------------------------------
-def pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, corr, pRefCell, pRefValue, cumulativeContErr ): 
-
-    U << rAU * UEqn.H()
-    if ( pimple.nCorr() <= 1 ):
+def pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, pRefCell, pRefValue, cumulativeContErr, sources ): 
+    U << rAU() * ( UEqn == sources( U ) ).H()
+    if ( pimple.nCorrPISO() <= 1 ):
        # UEqn.clear()
        pass
        
@@ -95,14 +95,14 @@ def pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, corr, pRefCell, pRefValue
     ref.adjustPhi( phi, U, p )
 
     # Non-orthogonal pressure corrector loop
-    for nonOrth in range( pimple.nNonOrthCorr() + 1):
+    while pimple.correctNonOrthogonal():
         #Pressure corrector
         pEqn = ref.fvm.laplacian( rAU, p ) == ref.fvc.div( phi )
         pEqn.setReference( pRefCell, pRefValue )
         
-        pEqn.solve( mesh.solver( p.select( pimple.finalInnerIter( corr, nonOrth ) ) ) )
+        pEqn.solve( mesh.solver( p.select( pimple.finalInnerIter() ) ) )
            
-        if ( nonOrth == pimple.nNonOrthCorr() ) :
+        if pimple.finalNonOrthogonalIter():
            phi -= pEqn.flux()
            pass
         pass
@@ -113,6 +113,7 @@ def pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, corr, pRefCell, pRefValue
 
     U -= rAU * ref.fvc.grad( p )
     U.correctBoundaryConditions()
+    sources.correct(U)
 
     return cumulativeContErr
 
@@ -126,7 +127,7 @@ def main_standalone( argc, argv ):
 
     mesh = man.createMesh( runTime )
 
-    p, U, phi, turbulence, pRefCell, pRefValue, laminarTransport = _createFields( runTime, mesh )
+    p, U, phi, turbulence, pRefCell, pRefValue, laminarTransport, sources = _createFields( runTime, mesh )
     
     cumulativeContErr = ref.initContinuityErrs()
     
@@ -146,23 +147,18 @@ def main_standalone( argc, argv ):
         ref.ext_Info() << "Time = " << runTime.timeName() << ref.nl << ref.nl
         
         # --- Pressure-velocity PIMPLE corrector loop
-        pimple.start()
         while pimple.loop():
-            if pimple.nOuterCorr() != 1 :
-               p.storePrevIter()
-               pass
+
+            UEqn, rAU = Ueqn( mesh, pimple, phi, U, p, turbulence, sources )
             
-            UEqn, rAU = Ueqn( mesh, pimple, phi, U, p, turbulence )
-            
-            # --- PISO loop
-            for corr in range( pimple.nCorr() ):
-               cumulativeContErr = pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, corr, pRefCell, pRefValue, cumulativeContErr )
+            # --- Pressure corrector loop
+            while pimple.correct():
+               cumulativeContErr = pEqn( runTime, mesh, pimple, U, rAU, UEqn, phi, p, pRefCell, pRefValue, cumulativeContErr, sources )
                pass
             
             if pimple.turbCorr():                             
                 turbulence.correct()
                 pass
-            pimple.increment()
             pass
 
         runTime.write();
@@ -180,7 +176,7 @@ def main_standalone( argc, argv ):
 
 #--------------------------------------------------------------------------------------
 from Foam import FOAM_VERSION
-if FOAM_VERSION( ">=", "020000" ):
+if FOAM_VERSION( ">=", "020100" ):
    if __name__ == "__main__" :
       import sys, os
       argv = sys.argv
@@ -188,7 +184,7 @@ if FOAM_VERSION( ">=", "020000" ):
       pass
    pass
 else:
-   ref.ext_Info() << "\n\n To use this solver it is necessary to SWIG OpenFOAM-2.0.0 or higher\n"
+   ref.ext_Info() << "\n\n To use this solver it is necessary to SWIG OpenFOAM-2.1.0 or higher\n"
    pass
 
 
